"""
Graphical user interface for bookfolder
"""

from bookfolder.gui.gui import GUI


def start():
    """
    Start the graphical user interface.
    """
    gui = GUI()
    gui.show()


if __name__ == "__main__":
    start()



"""
Graphical user interface for bookfolder
"""

import tkinter
from tkinter import N, E, S, W

from bookfolder.gui.widgets import (
    HorizontalSeparator,
    MeasurementIntervalFrame,
    PageNumberFrame,
    GeneratePatternFrame,
    ImageInput,
    PDFOutput,
    )


class GUI():
    """
    Graphical user interface for bookfolder
    """

    def __init__(self):
        self.image_path = ""

    def show(self):
        """
        Show the GUI to the user
        """
        window = tkinter.Tk()
        window.title("Bookfolding Pattern Creator")
        window.grid_columnconfigure(0, weight=1)

        outer_frame = tkinter.Frame()
        outer_frame.grid(padx="8p", pady="8p", sticky=(N, E, S, W))
        outer_frame.grid_columnconfigure(0, weight=1)

        image_input = ImageInput(outer_frame)
        pdf_output = PDFOutput(outer_frame)
        separator = HorizontalSeparator(outer_frame)
        measurement_interval_input = MeasurementIntervalFrame(outer_frame)
        page_number_input = PageNumberFrame(outer_frame)

        generate = GeneratePatternFrame(
            outer_frame,
            image_input.path,
            pdf_output.path,
            measurement_interval_input.measurement_interval,
            start_page=page_number_input.first_page_number
            )

        self._add_components_vertically(
            [
                image_input.frame,
                pdf_output.frame,
                separator.frame,
                measurement_interval_input.frame,
                page_number_input.frame,
                generate.frame,
            ])

        window.mainloop()

    def _add_components_vertically(self, components):
        """
        Add given `components` to a grid on top of each other.
        """
        # pylint: disable=no-self-use
        for index, component in enumerate(components):
            component.grid(row=index, column=0)



"""
Formatting tools for outputting the Sheet data in various forms.
"""


class Formatter():
    """
    Base class for classes outputting Sheet data in a specific format.
    """

    def __init__(self, sheets):
        self.sheets = sheets

    def format(self):
        """
        Return the formatted data for all the sheets.
        """
        raise NotImplementedError("Must be implemented in subclasses")


class CSVFormatter(Formatter):
    """
    Formatter that returns the data as a string CSV format.
    """

    def format(self, separator=";"):
        """
        Return the fold locations as a CSV.

        The default separator is a semicolon (;), but this can be overridden.

        The returned data is in the following format:
            page; cut and fold point locations (cm)
            1; 11.025; 11.975
            3; 10; 12.75
        """
        # pylint: disable=arguments-differ

        if len(separator) == 1:
            separator = separator + " "
        header = separator.join(["page",
                                 "cut and fold point locations (cm)"])
        csv_lines = [header]
        for sheet in self.sheets:
            column_data = [sheet.page_number] + sheet.fold_locations_in_cm()
            column_data_strs = [str(field) for field in column_data]
            csv_lines.append(separator.join(column_data_strs))
        return "\n".join(csv_lines)



"""
Tests for PatternCreator.
"""

from bookfolder.pattern_creator import PatternCreator

# pylint: disable=missing-function-docstring
# test functions should be self-documenting


def test_input_path():
    creator = PatternCreator("some/path.png")
    assert creator.input_path == "some/path.png"


def test_default_measurement_interval():
    creator = PatternCreator("some/path.png")
    assert creator.measurement_interval == 0.25


def test_measurement_interval_initialization():
    creator = PatternCreator("some/path.png", 1.0)
    assert creator.measurement_interval == 1.0


def test_extracts_correct_number_of_sheets():
    creator = PatternCreator("tests/data/simple_pattern.png")
    assert len(creator.sheets()) == 3


def test_extracted_sheets_have_correct_number_of_folds():
    creator = PatternCreator("tests/data/simple_pattern.png")
    assert len(creator.sheets()[0].folds) == 3
    assert len(creator.sheets()[1].folds) == 5
    assert len(creator.sheets()[2].folds) == 4


def test_extracted_sheets_have_correct_folds_locations():
    creator = PatternCreator("tests/data/simple_pattern.png")
    assert creator.sheets()[0].folds == [4, 6, 7]


def test_extracted_multicolor_sheets_have_correct_number_of_folds():
    creator = PatternCreator("tests/data/color_pattern.png")
    assert len(creator.sheets()[0].folds) == 5
    assert len(creator.sheets()[1].folds) == 5
    assert len(creator.sheets()[2].folds) == 4


def test_extracted_color_sheets_have_correct_folds_locations():
    creator = PatternCreator("tests/data/color_pattern.png")
    assert creator.sheets()[0].folds == [1, 3, 4, 6, 7]


def test_extracing_empty_column():
    creator = PatternCreator("tests/data/empty_column_pattern.png")
    assert creator.sheets()[1].folds == []


def test_default_start_page():
    creator = PatternCreator("tests/data/simple_pattern.png")
    assert creator.sheets()[0].page_number == 1


def test_setting_negative_start_page():
    creator = PatternCreator("tests/data/simple_pattern.png", start_page=-1)
    assert creator.sheets()[0].page_number == -1
    assert creator.sheets()[1].page_number == 1
    assert creator.sheets()[2].page_number == 3


def test_setting_positive_start_page():
    creator = PatternCreator("tests/data/simple_pattern.png", start_page=200)
    assert creator.sheets()[0].page_number == 200
    assert creator.sheets()[1].page_number == 202
    assert creator.sheets()[2].page_number == 204



"""
Representation of a sheet of paper in a book.
"""


class Sheet():
    """
    A sheet of book in a bookfolding sculpture.

    The locations of the folds are internally handled in "page coordinates",
    the separation of which is determined by `measurement_interval`.
    """

    def __init__(self, fold_locations, measurement_interval, page_number):
        """
        Create a new bookfolding sheet.

        :fold_locations: Locations of the folds on this page shown on page
                         coordinates.
        :measurement_interval: Length of a page coordinate unit in millimeters.
        """
        self.folds = fold_locations.copy()
        self.folds.sort()
        self.measurement_interval = measurement_interval
        self.page_number = page_number

    def fold_locations_in_mm(self):
        """
        Return a list of fold locations in millimeters.
        """
        decimals = self._decimal_places(self.measurement_interval)
        return [round(fold_index * self.measurement_interval, decimals)
                for fold_index in self.folds]

    def fold_locations_in_cm(self):
        """
        Return a list of fold locations in centimeters.
        """
        decimals = self._decimal_places(self.measurement_interval)
        return [round(fold_index * self.measurement_interval * 0.1, decimals+1)
                for fold_index in self.folds]

    def _decimal_places(self, float_number):
        """
        Return the number of digits after the decimal point.
        """
        # pylint: disable=no-self-use
        number_str = str(float_number)
        return number_str[::-1].find('.')



"""
Tests for Page class
"""

import pytest

from bookfolder.sheet import Sheet


# test function names should be self-documenting, i.e. often pretty long
# pylint: disable=missing-function-docstring
# pylint: disable=line-too-long


@pytest.fixture
def sheet_fx():
    """
    A test Sheet factory with overrideable defaults
    """
    def _sheet(folds=None, measurement_interval=0.1, page_number=1):
        if folds is None:
            folds = [1, 2, 3]
        return Sheet(folds, measurement_interval, page_number)
    return _sheet

# pylint doesn't understand fixtures
# pylint: disable=redefined-outer-name


@pytest.mark.parametrize(
    "folds",
    [
        pytest.param([], id="empty_folds_list"),
        pytest.param([1, 3, 100], id="short_folds_list"),
        pytest.param([100, 3, 1], id="short_unsorted_folds_list"),
        pytest.param(list(range(1000)), id="long_folds_list"),
    ]
)
def test_has_correct_folds(folds, sheet_fx):
    sheet = sheet_fx(folds)
    assert len(sheet.folds) == len(folds)
    assert set(sheet.folds) == set(folds)


def test_folds_are_sorted(sheet_fx):
    sheet = sheet_fx([1, 40, 2, 45, 3])
    assert sheet.folds == [1, 2, 3, 40, 45]


def test_fold_locations_in_mm(sheet_fx):
    sheet = sheet_fx([10, 25, 100, 101], measurement_interval=0.1)
    assert sheet.fold_locations_in_mm() == [1.0, 2.5, 10.0, 10.1]


def test_fold_locations_in_mm_decimals_affected_by_measument_interval_precision(sheet_fx):  # noqa:E501
    sheet = sheet_fx([10, 25, 100, 101], measurement_interval=0.25)
    assert sheet.fold_locations_in_mm() == [2.5, 6.25, 25.0, 25.25]


def test_fold_locations_in_cm(sheet_fx):
    sheet = sheet_fx([10, 25, 100, 101], measurement_interval=0.1)
    assert sheet.fold_locations_in_cm() == [0.1, 0.25, 1.0, 1.01]


def test_fold_locations_in_cm_decimals_affected_by_measument_interval_precision(sheet_fx):  # noqa:E501
    sheet = sheet_fx([10, 25, 100, 101], measurement_interval=0.25)
    assert sheet.fold_locations_in_cm() == [0.25, 0.625, 2.5, 2.525]



"""
Command line interface for the bookfolder app.
"""

import click

from bookfolder.formatter import CSVFormatter
from bookfolder.pattern_creator import PatternCreator


@click.group()
@click.argument("image", type=click.Path(exists=True))
@click.option("--measurement-interval",
              help="Height of one pixel in the IMAGE in millimeters",
              default=0.25,
              type=click.FLOAT)
@click.option("--start-page",
              help="The page number of the first sheet in the pattern",
              default=1,
              type=click.INT)
@click.pass_context
def cli(ctx, image, measurement_interval, start_page):
    """
    Generate a bookfolding pattern from IMAGE

    Each color of pixels in the image must correspond to one fold depth in the
    scuplture, with each horizontal pixel corresponding to one sheet of paper
    in the book and each vertical pixel to one MEASUREMENT_INTERVAL along the
    edge of the book.
    """
    ctx.ensure_object(dict)

    pattern_creator = PatternCreator(image, measurement_interval, start_page)
    sheets = pattern_creator.sheets()
    ctx.obj["sheets"] = sheets
    ctx.obj["pattern_creator"] = pattern_creator


@cli.command()
@click.pass_context
def csv_pattern(ctx):
    """
    Show the pattern in csv format
    """
    sheets = ctx.obj["sheets"]
    formatter = CSVFormatter(sheets)
    click.echo(formatter.format())


@cli.command()
@click.pass_context
@click.argument("output_file", type=click.Path())
def pdf_pattern(ctx, output_file):
    """
    Save the pattern as a PDF file
    """
    pattern_creator = ctx.obj["pattern_creator"]
    pattern_creator.save_pdf(output_file)
    click.echo("Wrote the pattern to {}".format(output_file))


if __name__ == "__main__":
    # pylint: disable=no-value-for-parameter
    # pylint: disable=unexpected-keyword-arg
    cli(obj={})



"""
Custom widgets for the bookfolder UI.
"""

import tkinter
from tkinter import filedialog
from tkinter import messagebox
from tkinter import ttk
from tkinter import N, E, S, W

from bookfolder.pattern_creator import PatternCreator


class WidgetFrame():
    """
    Superclass for widgets within the window.
    """

    x_padding = "4p"
    y_padding = "2p"
    frame_sticky = (N, E, S, W)

    def add_to_grid(self, component, column=0, row=0, padx=None, pady=None,
                    **kwargs):
        """
        Add given `component` to a grid layout.
        """
        # pylint: disable=too-many-arguments

        if padx is None:
            padx = self.x_padding
        if pady is None:
            pady = self.y_padding

        component.grid(column=column, row=row, padx=padx, pady=pady, **kwargs)


class HorizontalSeparator(WidgetFrame):
    """
    Frame containing a horizontal separator
    """

    y_padding = "5p"

    def __init__(self, parent_window):
        self._frame = ttk.Frame(parent_window)
        self._frame.grid(column=0, row=0, sticky=self.frame_sticky)

    @property
    def frame(self):
        """
        Generate the frame and the field within
        """
        separator = ttk.Separator(self._frame, orient="horizontal")
        self.add_to_grid(separator, column=0, row=0, sticky=(E, W))
        self._frame.columnconfigure(0, weight=1)
        return self._frame


class CustomizationOptionTextInputFrame(WidgetFrame):
    """
    Superclass for customization options set via a text input frame
    """
    frame_sticky = (N, E, S)
    field_width = 5


class MeasurementIntervalFrame(CustomizationOptionTextInputFrame):
    """
    Return a Frame with text box for entering page number
    """

    def __init__(self, parent_window):
        self._frame = ttk.Frame(parent_window)
        self._frame.grid(column=0, row=0, sticky=self.frame_sticky)
        self.measurement_interval = tkinter.DoubleVar(value=0.25)

    @property
    def frame(self):
        """
        Generate the frame and the field within
        """
        label = tkinter.Label(
            self._frame,
            text="Height of one pixel in mm")
        self.add_to_grid(label, column=0, row=0, sticky=(E))

        measurement_interval_entry = ttk.Entry(
                self._frame,
                textvariable=self.measurement_interval,
                width=self.field_width,
                )
        self.add_to_grid(measurement_interval_entry,
                         column=1, row=0, sticky=(E))

        return self._frame


class PageNumberFrame(CustomizationOptionTextInputFrame):
    """
    Return a Frame with text box for entering page number
    """

    def __init__(self, parent_window):
        self._frame = ttk.Frame(parent_window)
        self._frame.grid(column=0, row=0, sticky=self.frame_sticky)
        self.first_page_number = tkinter.IntVar(value=1)

    @property
    def frame(self):
        """
        Generate the frame and the field within
        """
        label = tkinter.Label(
            self._frame,
            text="Page number of first pattern sheet in the pattern")
        label.grid(column=0, row=0, sticky=(E))

        page_number_entry = ttk.Entry(
                self._frame,
                textvariable=self.first_page_number,
                width=self.field_width,
                )
        self.add_to_grid(page_number_entry,
                         column=1, row=0, sticky=(W))

        return self._frame


class GeneratePatternFrame(WidgetFrame):
    """
    Return a Frame with button to generate the pattern.
    """

    y_padding = ("10p", "3p")

    def __init__(self, parent_window, input_path, output_path,
                 measurement_interval, start_page):
        # pylint: disable=too-many-arguments
        self._frame = ttk.Frame(parent_window)
        self._frame.grid(column=0, row=0, sticky=self.frame_sticky)

        self.input_path = input_path
        self.output_path = output_path
        self.measurement_interval = measurement_interval
        self.start_page = start_page

    @property
    def frame(self):
        """
        Create the Frame and the button within.
        """
        btn = ttk.Button(self._frame, text="Generate",
                         command=self._generate)
        self.add_to_grid(btn, column=1, row=0, sticky=(E))
        self._frame.columnconfigure(0, weight=1)

        return self._frame

    def _generate(self):
        """
        Generate the PDF pattern and confirm creation to the user.
        """
        pattern_creator = PatternCreator(self.input_path.get(),
                                         self.measurement_interval.get(),
                                         self.start_page.get())
        pattern_creator.save_pdf(self.output_path.get())

        messagebox.showinfo(
            "Success",
            "Wrote the pattern to {}".format(self.output_path.get()))


class FileIOFrame(WidgetFrame):
    """
    Create a frame for selecting a file for input/output.
    """

    label_text = ""
    button_text = "Browse"

    def __init__(self, parent_window):
        self._frame = ttk.Frame(parent_window)
        self._frame.grid(column=0, row=0, sticky=self.frame_sticky)

        self.path = tkinter.StringVar()

    @property
    def frame(self):
        """
        Create the Frame and its components.

        :returns: the created Frame
        """

        label = tkinter.Label(self._frame, text=self.label_text, anchor=E)
        self.add_to_grid(label, column=0, row=0, sticky=(W, E))
        self._frame.columnconfigure(0, minsize="100p")

        image_textbox = ttk.Entry(self._frame, textvariable=self.path)
        self.add_to_grid(image_textbox, column=1, row=0, sticky=(E, W))

        btn_browse = ttk.Button(self._frame, text=self.button_text,
                                command=self._browse_files)
        self.add_to_grid(btn_browse, column=2, row=0, sticky=(W))

        self._frame.columnconfigure(1, weight=1)

        return self._frame

    def _browse_files(self):
        raise NotImplementedError("Must be implemented in subclasses")


class ImageInput(FileIOFrame):
    """
    Create a Frame for selecting source image for the pattern
    """

    label_text = "Source image"

    def _browse_files(self):
        filename = filedialog.askopenfilename(
            title="Select a File",
            filetypes=(
                ("Image files", "*.png"),
                ("all files", "*.*"),
                ),
            )

        self.path.set(filename)


class PDFOutput(FileIOFrame):
    """
    Create a Frame for selecting PDF output location
    """

    label_text = "Save as"

    def _browse_files(self):
        filename = filedialog.asksaveasfilename(
            title="Save as",
            filetypes=(
                ("PDF files", "*.pdf"),
                ("all files", "*.*"),
                ),
            )

        self.path.set(filename)



"""
Tool for extracting a bookfolding pattern from an image.
"""

from collections import namedtuple

from imageio import imread

from bookfolder.pdf import PDFWriter
from bookfolder.sheet import Sheet


InversionPoint = namedtuple("InversionPoint", "index from_color to_color")


class PatternCreator():
    """
    A tool for creating a bookfolding pattern from an image.

    The image must be black and white, with each horizontal pixel in the image
    corresponding to one sheet of paper in the book and each vertical pixel
    corresponding to one "page coordinate unit", i.e. the accuracy of the
    measurement tool used in the folding process.
    """

    def __init__(self, input_path, measurement_interval=0.25, start_page=1):
        """
        Initiate the PatternCreator.

        :input_path: Location of the input file containing the pattern image.
        :measurement_interval: Length of one page coordinate unit in
                               millimeters. Defaults to 0.25 mm.
        """
        self.input_path = input_path
        self.measurement_interval = measurement_interval
        self.start_page = start_page
        self._sheets = []

    def sheets(self):
        """
        Return `Sheet`s that make up this pattern
        """
        if not self._sheets:
            self._extract_sheets()
        return self._sheets

    def save_pdf(self, output_file):
        """
        Save the created pattern as a PDF file.
        """
        writer = PDFWriter(self.sheets())
        writer.create_document(["page", "measure, mark, cut and fold at (cm)"])
        writer.save(output_file)

    def _extract_sheets(self):
        """
        Read the image and set `self._sheets` accordingly.
        """
        image_data = imread(self.input_path, pilmode="RGB")
        for column_index in range(image_data.shape[1]):
            inversions = self._inversion_points(image_data[:, column_index, :])
            self._sheets.append(
                Sheet(
                    [inversion.index for inversion in inversions],
                    self.measurement_interval,
                    page_number=(column_index * 2 + self.start_page)
                    )
                )

    def _inversion_points(self, image_column):
        """
        Return information about color changes as a list of `InversionPoint`s

        The resulting `InversionPoint`s each have the following attributes:
         - `index`: the index of the pixel whose color differs from the one
                    before it
         - `from_color`: color of the previous pixel
         - `to_color`: the color of the latter pixel

        :image_column: A list of pixels colors
        """
        # pylint: disable=no-self-use
        inversion_points = []
        for i in range(1, len(image_column)):
            if (image_column[i-1] != image_column[i]).any():
                inversion_points.append(InversionPoint(i,
                                                       image_column[i-1],
                                                       image_column[i]))
        return inversion_points



"""
Module for writing the pattern into a PDF file.
"""

import math

from fpdf import FPDF


class PDFWriter():
    """
    A tool that writes bookfolding Sheet data into a file.
    """

    font = "Times"
    font_size = 10

    line_height = font_size * 1.0
    n_columns = 14

    header_height = 0
    footer_height = 0
    rows_per_page = 26

    def __init__(self, sheets):
        """
        Create a `PDFWriter`.

        :sheets: a list of Sheet objects depicting pages in the bookfolding
                 pattern.
        """
        self.sheets = sheets
        self.pdf = FPDF()
        self.pdf.set_font(self.font, size=self.font_size)
        self.column_width = self.pdf.epw / self.n_columns

    def create_document(self, table_header):
        """
        Write all `Sheet`s into a bookfolding pattern document.

        The resulting document can have multiple pages if the pattern is long.
        """
        for pageful_of_sheets in self._sheet_pages():
            self._add_table_page(table_header)
            for sheet in pageful_of_sheets:
                self._add_rows_for_sheet(sheet)

    def save(self, output_path):
        """
        Write the constructed PDF into a file.

        :output_path: the file in which the PDF file is saved.
        """
        self.pdf.output(output_path)

    def _sheet_pages(self):
        """
        Iterate over lists of sheets, each with one pageful of folds.

        :yields: lists of sheets from self.sheets, one pageful at a time
        """
        start_index = 0
        while True:

            sheets = []
            rows_required = 0

            for sheet in self.sheets[start_index:]:
                new_rows_required = math.ceil(
                    len(sheet.folds) / (self.n_columns - 1))
                if rows_required + new_rows_required > self.rows_per_page - 1:
                    break
                sheets.append(sheet)
                rows_required += new_rows_required

            start_index += len(sheets)

            if not sheets:
                return
            yield sheets

    def _add_table_page(self, table_header):
        """
        Add a new page to the document and add a table header to that page.

        :table_header: List of header fields.
        """
        self.pdf.add_page()
        self._add_header(table_header)
        self._add_new_row()

    def _add_header(self, table_header):
        """
        Write a header to the current page
        """
        if len(table_header) < self.n_columns:

            for i in range(len(table_header) - 1):
                self._add_cell(table_header[i], border=1)

            last_column_width = (
                    (self.n_columns - len(table_header) + 1)
                    * self.column_width
                    )
            self._add_cell(table_header[-1], width=last_column_width)

        elif len(table_header) == self.n_columns:
            for cell in table_header:
                self._add_cell(cell, border=1)

        else:
            raise ValueError("Header {} has too many fields: the page only "
                             "fits {} fields"
                             "".format(table_header, self.n_columns))

    def _add_rows_for_sheet(self, sheet):
        """
        Add one or more data rows representing a single `Sheet`.
        """
        rows_required = self._rows_required(sheet)

        for current_row_index, fold_indices \
                in enumerate(self._iterate_fold_rows(sheet)):
            self._add_page_number_cell(sheet,
                                       current_row_index,
                                       rows_required)
            fold_points_for_row = [
                sheet.fold_locations_in_cm()[i] for i in fold_indices
                ]

            for fold_point in fold_points_for_row:
                self._add_cell(fold_point)
            self._add_n_empty_cells(
                self.n_columns - len(fold_points_for_row) - 1)
            self._add_new_row()

    def _rows_required(self, sheet):
        """
        Return the number of rows that are needed to represent the `sheet`
        """
        n_datum_per_row = self.n_columns - 1
        return math.ceil(len(sheet.folds) / n_datum_per_row)

    def _iterate_fold_rows(self, sheet):
        """
        Iterate over rows of fold indices.

        Each row is returned as a list of indices for the folds in `sheet` that
        are to be written on that row.

        Each row has room for `self.n_columns` cells, one of which is reserved
        for the page number.
        """
        n_datum_per_row = self.n_columns - 1
        for i in range(0, len(sheet.folds), n_datum_per_row):
            yield range(
                    i,
                    min(i + n_datum_per_row, len(sheet.folds))
                    )

    def _add_page_number_cell(self, sheet, current_row_index, total_rows):
        """
        Add a page number cell for `current_row_index`th row of folds in
        `sheet`.

        The first row for each sheet contains the page number, while the other
        page number cells are empty. All page number cells have left and right
        borders, but only the topmost page number cell has top border and the
        bottom cell has bottom border.

        :sheet: `Sheet` for which the cell is added
        :current_row_index: The index of the row for which the page number cell
                            is to be added
        :total_rows: Total number of rows required for this page
        """
        if total_rows == 1:
            borders = "TBLR"
            text = sheet.page_number
        elif current_row_index == 0:
            borders = "TLR"
            text = sheet.page_number
        elif current_row_index == total_rows - 1:
            borders = "BLR"
            text = ""
        else:
            borders = "LR"
            text = ""
        self._add_cell(text, border=borders)

    def _add_cell(self, content, border=1, width=None):
        """
        Add a table cell to the PDF.

        :content: Contents of the cell
        :border: Which borders to draw. 1 for all borders, 0 for none, or a
                 string containing a set of characters from L, R, T and B for
                 left, right, top and bottom borders respectively. Defaults to
                 1.
        :width: Width of the cell. Defaults to self.column_width.
        """
        if width is None:
            width = self.column_width

        self.pdf.multi_cell(
            width,
            self.line_height,
            str(content),
            border=border,
            ln=3,
            max_line_height=self.pdf.font_size
            )

    def _add_n_empty_cells(self, n_cells):
        """
        Add `n_cells` empty cells.
        """
        for _ in range(n_cells):
            self._add_cell("")

    def _add_new_row(self):
        """
        Add new row to the table
        """
        self.pdf.ln(self.line_height)



from os import path
from setuptools import setup, find_packages

current_directory = path.abspath(path.dirname(__file__))
with open(path.join(current_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="bookfolder",
    version="0.1",
    author="Anni Järvenpää",
    description="Convert an image into a bookfolding pattern",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/aajarven/bookfolder",
    packages=find_packages(),
    python_requires=">=3.6",
    include_package_data=True,
    install_requires=[
        "click",
        "imageio",
    ],
    entry_points={
        "console_scripts": [
            "bookfolder-cli = bookfolder.scripts.cli:cli",
            "bookfolder = bookfolder.scripts.gui:start",
        ],
    },
)



"""
Tests for formatter classes
"""

import pytest

from bookfolder.formatter import CSVFormatter
from bookfolder.sheet import Sheet


@pytest.fixture
def simple_sheets_fx():
    """
    Return a list of three Sheet objects.
    """
    measurement_interval = 0.1
    return [
        Sheet([10, 15, 20],
              measurement_interval=measurement_interval,
              page_number=1),
        Sheet([12, 14, 25],
              measurement_interval=measurement_interval,
              page_number=3),
        Sheet([13, 14, 30],
              measurement_interval=measurement_interval,
              page_number=5),
        ]


# tests should have descriptive enough names
# pylint: disable=missing-function-docstring

# pylint doesn't recognize fixtures
# pylint: disable=redefined-outer-name

def test_csv_header_present():
    formatter = CSVFormatter([])
    assert "page; cut and fold point locations (cm)" in formatter.format()


def test_csv_formatter_produces_right_number_of_lines(simple_sheets_fx):
    formatter = CSVFormatter(simple_sheets_fx)
    assert len(formatter.format().split("\n")) == 4


def test_csv_formatter_produces_right_data_contents(simple_sheets_fx):
    formatter = CSVFormatter(simple_sheets_fx)
    assert formatter.format() == (
        "page; cut and fold point locations (cm)\n"
        "1; 0.1; 0.15; 0.2\n"
        "3; 0.12; 0.14; 0.25\n"
        "5; 0.13; 0.14; 0.3")
